/**
 * Created April 23, 2022
 * 
 * This work is a part of ESP Signer library
 * Copyright (c) 2022, K. Suwatchai (Mobizt)
 * 
 * The MIT License (MIT)
 * Copyright (c) 2022, K. Suwatchai (Mobizt)
 * 
 * 
 * Permission is hereby granted, free of charge, to any person returning a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef ESP_SIGNER_HTTP_CODE_H_
#define ESP_SIGNER_HTTP_CODE_H_

/// HTTP client errors
#define ESP_SIGNER_ERROR_TCP_ERROR_CONNECTION_REFUSED (-1)
#define ESP_SIGNER_ERROR_TCP_ERROR_SEND_HEADER_FAILED (-2)
#define ESP_SIGNER_ERROR_TCP_ERROR_SEND_PAYLOAD_FAILED (-3)
#define ESP_SIGNER_ERROR_TCP_ERROR_NOT_CONNECTED (-4)
#define ESP_SIGNER_ERROR_TCP_ERROR_CONNECTION_LOST (-5)
#define ESP_SIGNER_ERROR_TCP_ERROR_NO_STREAM (-6)
#define ESP_SIGNER_ERROR_TCP_ERROR_NO_HTTP_SERVER (-7)
#define ESP_SIGNER_ERROR_TCP_ERROR_TOO_LESS_RAM (-8)
#define ESP_SIGNER_ERROR_TCP_ERROR_ENCODING (-9)
#define ESP_SIGNER_ERROR_TCP_ERROR_STREAM_WRITE (-10)
#define ESP_SIGNER_ERROR_TCP_RESPONSE_PAYLOAD_READ_TIMED_OUT (-11)
#define ESP_SIGNER_ERROR_BUFFER_OVERFLOW (-12)
#define ESP_SIGNER_ERROR_TCP_ERROR_CONNECTION_INUSED -13
#define ESP_SIGNER_ERROR_CANNOT_CONFIG_TIME -14
#define ESP_SIGNER_ERROR_SSL_RX_BUFFER_SIZE_TOO_SMALL -15
#define ESP_SIGNER_ERROR_FILE_IO_ERROR -16
#define ESP_SIGNER_ERROR_FILE_NOT_FOUND -17
#define ESP_SIGNER_ERROR_UNINITIALIZED -18
#define ESP_SIGNER_ERROR_TOKEN_SET_TIME -19
#define ESP_SIGNER_ERROR_TOKEN_CREATE_HASH -20
#define ESP_SIGNER_ERROR_TOKEN_PARSE_PK -21
#define ESP_SIGNER_ERROR_TOKEN_SIGN -22
#define ESP_SIGNER_ERROR_TOKEN_EXCHANGE -23
#define ESP_SIGNER_ERROR_TOKEN_EXCHANGE_MAX_RETRY_REACHED -24
#define ESP_SIGNER_ERROR_TOKEN_NOT_READY -25
#define ESP_SIGNER_ERROR_NTP_SYNC_TIMED_OUT -26

#define ESP_SIGNER_ERROR_HTTP_CODE_UNDEFINED -1000

/// HTTP codes see RFC7231

#define ESP_SIGNER_ERROR_HTTP_CODE_OK 200
#define ESP_SIGNER_ERROR_HTTP_CODE_NON_AUTHORITATIVE_INFORMATION 203
#define ESP_SIGNER_ERROR_HTTP_CODE_NO_CONTENT 204
#define ESP_SIGNER_ERROR_HTTP_CODE_MOVED_PERMANENTLY 301
#define ESP_SIGNER_ERROR_HTTP_CODE_FOUND 302
#define ESP_SIGNER_ERROR_HTTP_CODE_USE_PROXY 305
#define ESP_SIGNER_ERROR_HTTP_CODE_TEMPORARY_REDIRECT 307
#define ESP_SIGNER_ERROR_HTTP_CODE_PERMANENT_REDIRECT 308
#define ESP_SIGNER_ERROR_HTTP_CODE_BAD_REQUEST 400
#define ESP_SIGNER_ERROR_HTTP_CODE_UNAUTHORIZED 401
#define ESP_SIGNER_ERROR_HTTP_CODE_FORBIDDEN 403
#define ESP_SIGNER_ERROR_HTTP_CODE_NOT_FOUND 404
#define ESP_SIGNER_ERROR_HTTP_CODE_METHOD_NOT_ALLOWED 405
#define ESP_SIGNER_ERROR_HTTP_CODE_NOT_ACCEPTABLE 406
#define ESP_SIGNER_ERROR_HTTP_CODE_PROXY_AUTHENTICATION_REQUIRED 407
#define ESP_SIGNER_ERROR_HTTP_CODE_REQUEST_TIMEOUT 408
#define ESP_SIGNER_ERROR_HTTP_CODE_LENGTH_REQUIRED 411
#define ESP_SIGNER_ERROR_HTTP_CODE_PRECONDITION_FAILED 412
#define ESP_SIGNER_ERROR_HTTP_CODE_PAYLOAD_TOO_LARGE 413
#define ESP_SIGNER_ERROR_HTTP_CODE_URI_TOO_LONG 414
#define ESP_SIGNER_ERROR_HTTP_CODE_MISDIRECTED_REQUEST 421
#define ESP_SIGNER_ERROR_HTTP_CODE_UNPROCESSABLE_ENTITY 422
#define ESP_SIGNER_ERROR_HTTP_CODE_TOO_MANY_REQUESTS 429
#define ESP_SIGNER_ERROR_HTTP_CODE_REQUEST_HEADER_FIELDS_TOO_LARGE 431
#define ESP_SIGNER_ERROR_HTTP_CODE_INTERNAL_SERVER_ERROR 500
#define ESP_SIGNER_ERROR_HTTP_CODE_NOT_IMPLEMENTED 501
#define ESP_SIGNER_ERROR_HTTP_CODE_BAD_GATEWAY 502
#define ESP_SIGNER_ERROR_HTTP_CODE_SERVICE_UNAVAILABLE 503
#define ESP_SIGNER_ERROR_HTTP_CODE_GATEWAY_TIMEOUT 504
#define ESP_SIGNER_ERROR_HTTP_CODE_HTTP_VERSION_NOT_SUPPORTED 505
#define ESP_SIGNER_ERROR_HTTP_CODE_LOOP_DETECTED 508
#define ESP_SIGNER_ERROR_HTTP_CODE_NETWORK_AUTHENTICATION_REQUIRED 511

#endif